/**   -*- Mode: java4; -*-
 * Licensed under the Common Development and Distribution License,
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *   http://www.sun.com/cddl/
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or
 * implied. See the License for the specific language governing
 * permissions and limitations under the License.
 *
 *
 * "Portions Copyrighted [2007] [MASAHITO HENMI]"
 */

package com.sun.facelets.tag.jstl.core;

import java.io.IOException;
import java.lang.reflect.Array;
import java.util.Collection;
import java.util.Iterator;
import java.util.Map;

import javax.el.ELException;
import javax.el.ValueExpression;
import javax.el.VariableMapper;
import javax.faces.FacesException;
import javax.faces.component.UIComponent;

import com.sun.facelets.FaceletContext;
import com.sun.facelets.FaceletException;
import com.sun.facelets.tag.TagAttribute;
import com.sun.facelets.tag.TagAttributeException;
import com.sun.facelets.tag.TagConfig;
import com.sun.facelets.tag.TagHandler;
import java.util.logging.*;

/**
 * @author Jacob Hookom
 * @author Andrew Robinson
 * @version $Id: ForEachHandler.java,v 1.8 2006/03/19 03:11:34 jhook Exp $
 */
public final class ForEachHandler extends TagHandler {

    protected final static Logger log = Logger.getLogger("strawberry.tag");

    private static class ArrayIterator implements Iterator {

        protected final Object array;

        protected int i;

        protected final int len;

        public ArrayIterator(Object src) {
            this.i = 0;
            this.array = src;
            this.len = Array.getLength(src);
            log.fine("ArrayIteratorRXgN^ len = " + len);
        }

        public boolean hasNext() {
            return this.i < this.len;
        }

        public Object next() {
            Object wk = Array.get(this.array, this.i++);
            log.fine("ArrayIterator#next => " + wk);
            return wk;
        }

        public void remove() {
            throw new UnsupportedOperationException();
        }
    }

    private final TagAttribute begin;

    private final TagAttribute end;

    private final TagAttribute items;

    private final TagAttribute step;

    private final TagAttribute tranzient;

    private final TagAttribute var;

    private final TagAttribute varStatus;

    /**
     * @param config
     */
    public ForEachHandler(TagConfig config) {
        super(config);
        this.items = this.getAttribute("items");
        this.var = this.getAttribute("var");
        this.begin = this.getAttribute("begin");
        this.end = this.getAttribute("end");
        this.step = this.getAttribute("step");
        this.varStatus = this.getAttribute("varStatus");
        this.tranzient = this.getAttribute("transient");

        if (this.items == null && this.begin != null && this.end == null) {
            throw new TagAttributeException(
                this.tag,
                this.begin,
                "If the 'items' attribute is not specified, but the 'begin' attribute is, then the 'end' attribute is required");
        }
    }

    public void apply(FaceletContext ctx, UIComponent parent)
      throws IOException, FacesException, FaceletException, ELException {

        //log.fine("apply " + Integer.toHexString(ctx.hashCode()));

        if (this.items != null) {
            applyItems(ctx, parent);
        } else {
            applyNoItems(ctx, parent);
        }
    }

    public void applyItems(FaceletContext ctx, UIComponent parent)
      throws IOException, FacesException, FaceletException, ELException {

        int s = this.getBegin(ctx);
        int e = this.getEnd(ctx);
        int m = this.getStep(ctx);
        log.fine("(120) Begin = " + s + ", End = " + e + ", Step = " + m);
        Integer sO = this.begin != null ? new Integer(s) : null;
        Integer eO = this.end != null ? new Integer(e) : null;
        Integer mO = this.step != null ? new Integer(m) : null;

        boolean t = this.getTransient(ctx);
        Object src = null;
        ValueExpression srcVE = null;
        if (this.items != null) {
            srcVE = this.items.getValueExpression(ctx, Object.class);
            src = srcVE.getValue(ctx); // Œl𓾂i]j
        }
        if (src != null) {
            Iterator itr = this.toIterator(src);
            if (itr != null) {
                int i = 0;

                /* move to start*/
                while (i < s && itr.hasNext()) {
                    itr.next();
                    i++;
                }

                String v = this.getVarName(ctx);
                String vs = this.getVarStatusName(ctx);
                VariableMapper vars = ctx.getVariableMapper();
                ValueExpression ve = null;
                //log.fine("(155) ************************** captureJn "  );
                ValueExpression vO = this.capture(v, vars);
                ValueExpression vsO = this.capture(vs, vars);
                //log.fine("(158) ************************** capture ..."  );
                int mi = 0;
                Object value = null;
                try {
                    boolean first = true;
                    while (i <= e && itr.hasNext()) {
                        value = itr.next();
                        log.fine("(165) ******** " + ((value == null) ? "null" : value.getClass().getName()));
                        // set the var
                        if (v != null) {
                            if (t || srcVE == null) {
                                ctx.setAttribute(v, value);
                            } else {
                                ve = this.getVarExpr(srcVE, src, value, i);//FValueExpĒlZ[u
                                vars.setVariable(v, ve);
                            }
                        }

                        // set the varStatus
                        if (vs != null) {
                            IterationStatus itrS = new IterationStatus(first, !itr.hasNext(),i, sO, eO, mO);
                            if (t || srcVE == null) {
                                ctx.setAttribute(vs, itrS);
                            } else {
                                ve = new IterationStatusExpression(itrS);
                                vars.setVariable(vs, ve);
                            }
                        }

                        // execute body
                        this.nextHandler.apply(ctx, parent);

                        // increment steps
                        mi = 1;
                        while (mi < m && itr.hasNext()) {
                            itr.next();
                            mi++;
                            i++;
                        }
                        i++;

                        first = false;
                    }
                } finally {
                    if (v != null) {
                        vars.setVariable(v, vO);
                    }
                    if (vs != null) {
                        vars.setVariable(vs, vsO);
                    }
                    //log.fine("(208) ************************** captureI "  );
                }
            }
        }
    }

    public void applyNoItems(FaceletContext ctx, UIComponent parent)
      throws IOException, FacesException, FaceletException, ELException {

        int s = this.getBegin(ctx);
        int e = this.getEnd(ctx);
        int m = this.getStep(ctx);
        log.fine("(225) Begin = " + s + ", End = " + e + ", Step = " + m);
        Integer sO = this.begin != null ? new Integer(s) : null;
        Integer eO = this.end != null ? new Integer(e) : null;
        Integer mO = this.step != null ? new Integer(m) : null;

        boolean t = this.getTransient(ctx);

        ValueExpression srcVE = null;
        if (this.items != null)  throw new IllegalStateException();
        if (m == 0) throw new IllegalArgumentException();

        int i = s;

        String v = this.getVarName(ctx);
        String vs = this.getVarStatusName(ctx);
        VariableMapper vars = ctx.getVariableMapper();
        ValueExpression ve = null;
        log.fine("(241) ************************** captureJn "  );
        ValueExpression vO = this.capture(v, vars);
        ValueExpression vsO = this.capture(vs, vars);
        log.fine("(244) ************************** capture ..."  );
        int mi = 0;
        Object value = null;
        try {
            boolean first = true;
            while ((m > 0 && i <= e) || (m < 0 &&  i >= e)) {
                log.fine("(250) **************************  i = " + i);
                value = new Integer(i);
                // set the var
                if (v != null) {
                    if (t || srcVE == null) {
                        log.fine("(255) ************************** " + value);
                        ctx.setAttribute(v, value);
                    }
                }
                int nextI = i + m;
                boolean flag = (m > 0 && nextI <= e) || (m < 0 &&  nextI >= e);
                // set the varStatus
                if (vs != null) {
                    log.fine("(263) ******** first = " + value + ", last = " + !flag);
                    IterationStatus itrS = new IterationStatus(first, !flag,i, sO, eO, mO);
                    if (t || srcVE == null) {
                        ctx.setAttribute(vs, itrS);
                    }
                }
                // execute body
                this.nextHandler.apply(ctx, parent);
                // increment steps
                i+= m;
                first = false;
            }
        } finally {
            if (v != null) {
                vars.setVariable(v, vO);
            }
            if (vs != null) {
                vars.setVariable(vs, vsO);
            }
            log.fine("(282) ************************** captureI "  );
        }
    }
    private final ValueExpression capture(String name, VariableMapper vars) {
        //log.fine("#capture, VariableMapperNext6Ⴞ߂" + vars);
        if (name != null) {
            return vars.setVariable(name, null);
        }
        return null;
    }

    private final int getBegin(FaceletContext ctx) {
        if (this.begin != null) {
            return this.begin.getInt(ctx);
        }
        return 0;
    }

    private final int getEnd(FaceletContext ctx) {
        if (this.end != null) {
            return this.end.getInt(ctx);
        }
        return Integer.MAX_VALUE;
    }

    private final int getStep(FaceletContext ctx) {
        if (this.step != null) {
            return this.step.getInt(ctx);
        }
        return 1;
    }

    private final boolean getTransient(FaceletContext ctx) {
        if (this.tranzient != null) {
            return this.tranzient.getBoolean(ctx);
        }
        return false;
    }

    private final ValueExpression getVarExpr(ValueExpression ve, Object src,
                                             Object value, int i) {
        /*
        if (src instanceof Collection || src.getClass().isArray()) {
            return new IndexedValueExpression(ve, i);
        } else if (src instanceof Map && value instanceof Map.Entry) {
            return new MappedValueExpression(ve, (Map.Entry) value);
        }
        throw new IllegalStateException("Cannot create VE for: " + src);*/
        FValueExpression ans = new FValueExpression();
        ans.setGetValueValue(value);
        return ans;
    }

    private final String getVarName(FaceletContext ctx) {
        if (this.var != null) {
            return this.var.getValue(ctx);
        }
        return null;
    }

    private final String getVarStatusName(FaceletContext ctx) {
        if (this.varStatus != null) {
            return this.varStatus.getValue(ctx);
        }
        return null;
    }

    private final Iterator toIterator(Object src) {
        if (src == null) {
            return null;
        } else if (src instanceof Collection) {
            return ((Collection) src).iterator();
        } else if (src instanceof Map) {
            return ((Map) src).entrySet().iterator();
        } else if (src.getClass().isArray()) {
            return new ArrayIterator(src);
        } else {
            throw new TagAttributeException(this.tag, this.items,
                                            "Must evaluate to a Collection, Map, Array, or null.");
        }
    }

}
